#! /usr/bin/env python
# -*- coding: utf-8 -*-

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   The GPL v3 licence is available at: https://www.gnu.org/licenses/gpl-3.0.en.html

# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial release. Inspired by Rich2005 in post:
# https://www.gimp-forum.net/Thread-Polygonal-designs-and-intersections?highlight=unicode
# Rel 0.2: Add repeating the character
# Rel 0.3: Add justification
# Rel 0.4: Add use to Map or custom text,add option "Centered".
# Rel 0.5: Add options: create new image, fill pattern .

from gimpfu import *
import math

def python_unicode_map_to_gimp(img, layer, create_new, fill, color, pattern, size, code, font , repeteat, many, margin, wx, hy, letter_spacing, line_spacing, centered, flip, rotate, angle, crop):

################################### Rel 0.5 #############################################

	if create_new==1:		
		img = pdb.gimp_image_new(size, size, 0)
		display = pdb.gimp_display_new(img)
		drawable = pdb.gimp_layer_new(img, size, size, 1, "Background", 100, 0)
		pdb.gimp_image_insert_layer(img, drawable, None, 0)
		
########################################################################################

	pdb.gimp_image_undo_group_start(img)
	pdb.gimp_context_push()
	
	code = code.decode("utf-8")
	fontname = str(code)
	name = font

	w = pdb.gimp_image_width(img)
	h = pdb.gimp_image_height(img)
	name = fontname	
	if repeteat==0:
		fn = fontname
	if repeteat==1:
		fn = many * fontname
	if repeteat==2:
		fn = many * (fontname + '\n')

	add = gimp.Layer(img, "", w, h, RGBA_IMAGE, 100, 0)
	img.add_layer(add, -1)
	layer.add_alpha()

	pdb.gimp_edit_clear(add)
	pdb.gimp_selection_none(img)
	tlayer = pdb.gimp_text_fontname(img, add, wx, hy, fn, margin, TRUE, size, PIXELS, font)

	# justification:
	pdb.gimp_text_layer_set_letter_spacing(tlayer, letter_spacing)
	pdb.gimp_text_layer_set_line_spacing(tlayer, line_spacing)
	
	if centered==1:
		wh = pdb.gimp_drawable_width(tlayer)
		hh = pdb.gimp_drawable_height(tlayer)
		centerx=w/2
		centery=h/2

		adw=wh/2
		adh=hh/2
		sw=centerx - adw
		sh=centery - adh		
		pdb.gimp_layer_set_offsets(tlayer, sw, sh)
	
	# vectors	
	new_vectors = pdb.gimp_vectors_new(img, tlayer)
	x=pdb.gimp_vectors_new_from_text_layer(img, tlayer)
	pdb.gimp_image_insert_vectors(img, x, None, -1)
	active_vectors = pdb.gimp_image_get_active_vectors(img)
	active_vectors.name = active_vectors.name.replace(fontname, name)
	active_vectors.visible = True
	
	if flip==1:
		pdb.gimp_item_transform_flip_simple(tlayer, 0, FALSE, w/2)
		pdb.gimp_item_transform_flip_simple(active_vectors, 0, TRUE,0)		
	if flip==2:
		pdb.gimp_item_transform_flip_simple(tlayer, 1, FALSE, h/2)
		pdb.gimp_item_transform_flip_simple(active_vectors, 1, TRUE,0)
		
	if rotate==1:
		pdb.gimp_item_transform_rotate_simple(tlayer, 0, FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate_simple(active_vectors, 0, TRUE,0,0)		
	if rotate==2:
		pdb.gimp_item_transform_rotate_simple(tlayer, 1, FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate_simple(active_vectors, 1, TRUE,0,0)		
	if rotate==3:
		pdb.gimp_item_transform_rotate_simple(tlayer, 2, FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate_simple(active_vectors, 2, TRUE,0,0)
	if rotate==4:		
		pdb.gimp_item_transform_rotate(tlayer, angle*0.01745329252 ,FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate(active_vectors, angle*0.01745329252, TRUE,0,0)		
		
	pdb.gimp_floating_sel_anchor(tlayer)
	pdb.gimp_item_set_name(add, name)
	pdb.gimp_image_raise_item_to_top(img, add)
	pdb.gimp_image_resize_to_layers(img)	
	# Crop
	if crop==1:
		pdb.gimp_selection_none(img)
		pdb.plug_in_autocrop_layer(img, add)
		
	if fill==1:
		pdb.gimp_image_select_item(img,0,add)
		pdb.gimp_context_set_background(color)
		pdb.gimp_edit_fill(add,1)
		
	if fill==2:
		pdb.gimp_image_select_item(img,0,add)
		pdb.gimp_context_set_pattern(pattern)
		pdb.gimp_edit_fill(add,4) 
		
	pdb.gimp_selection_none(img)
	pdb.gimp_item_set_visible(layer, TRUE)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(img)
		
register(
	"python_unicode_map_to_gimp",
	" "u'\u2648'" UNICODE Map to GIMP "u'\u264e'"  ",
	"Unicode to shape and paths V.0.5",
	"MareroQ",
	"GPL",
	"2019",
	"Map (text) to GIMP...",
	"*",
	[
	(PF_IMAGE, "image", "Input image", None), #current image
	(PF_DRAWABLE, "layer", "Input layer", None), #current layer
	(PF_BOOL,  "create_new", "Create new image:", False),	
	(PF_OPTION, "fill", "Filling the Text", 0,(" DON'T USE ", "Color","Patterns")),
	(PF_COLOR, "color", "Text color:", (0, 0, 0)),	
	(PF_PATTERN, "pattern", "Text pattern", "Metal Grit-1"),		
	(PF_INT,    "size", "Size: ", 300          ),
	(PF_TEXT, 	"code", "Custom Text\n( or paste a Unicode\n character):", ""),
	(PF_FONT, "font", "Custom Font", "Arial"),
	(PF_OPTION,"repeteat", "Repeteat ? :", 0, ["Do not repeat","Horizontally "u'\u21e8'"","Vertical "u'\u21e9'""]),	
	(PF_INT,    "many", "Repeat Sign: ", 2),
	(PF_SPINNER, "margin", "Add Margin x=y (px): ", 0, (0, 5000, 1)),
	(PF_SPINNER, "wx", "Offset x (px) "u'\u21e8'": ", 0, (-1000, 5000, 1)),	
	(PF_SPINNER, "hy", "Offset y (px) "u'\u21e9'": ", 0, (-1000, 5000, 1)),	
	(PF_SPINNER, "letter_spacing", "Horizontal Spacing: ", 0, (-1000, 1000, 1, 5, 0, 0)),
	(PF_SPINNER, "line_spacing", "Vertical Spacing: ", 0, ( -1000, 1000, 1, 5, 0, 0)),
	(PF_BOOL,    "centered", "Centered?", True),			
	(PF_OPTION,"flip", "Mirror ? (auto-center) :", 0, ["Not mirror","HORIZONTAL "u'\u21e8'"", "VERTICAL "u'\u21e9'""]),	
	(PF_OPTION,"rotate", "Rotate ? (auto-center) :", 0, ["Not rotate","ROTATE-90", "ROTATE-180", "ROTATE-270","CUSTOM ANGLE  "u'\u21bb'""]),
	(PF_SPINNER, "angle", "Custom Angle: ", 0, (1, 360, 1)),
	(PF_BOOL,    "crop", "Crop to empty borders?", True),	
	],
	[],
	python_unicode_map_to_gimp,
	menu="<Image>/Filters/Path/"u'\u25cf'" Unicode to shape/",
	)
main()